import React, { useState, useEffect } from 'react'
import { Link, useLocation, useParams } from 'react-router-dom'
import SettingsDialogue from './SettingsDialogue'
import { Button } from './ui/button'
import clsx from 'clsx'

type User = { id:               number
              name:             string | null
              inheritanceCode:  string
              role:             string }

type Props = { user:    User
               setUser: (user: User) => void }

const enum Menu { None,
                  Post,
                  Deerjikist,
                  Tag,
                  Wiki }


const TopNav: React.FC = ({ user, setUser }: Props) => {
  const location = useLocation ()

  const [settingsVisible, setSettingsVisible] = useState (false)
  const [selectedMenu, setSelectedMenu] = useState<Menu> (Menu.None)

  const MyLink = ({ to, title, menu, base }: { to:    string
                                               title: string
                                               menu?: Menu
                                               base?: string }) => (
      <Link to={to} className={clsx ('hover:text-orange-500 h-full flex items-center',
                                     (location.pathname.startsWith (base ?? to)
                                      ? 'bg-gray-700 px-4 font-bold'
                                      : 'px-2'))}>
        {title}
      </Link>)

  useEffect (() => {
    if (location.pathname.startsWith ('/posts'))
      setSelectedMenu (Menu.Post)
    else if (location.pathname.startsWith ('/deerjikists'))
      setSelectedMenu (Menu.Deerjikist)
    else if (location.pathname.startsWith ('/tags'))
      setSelectedMenu (Menu.Tag)
    else if (location.pathname.startsWith ('/wiki'))
      setSelectedMenu (Menu.Wiki)
    else
      setSelectedMenu (Menu.None)
  }, [location])

  return (
      <>
        <nav className="bg-gray-800 text-white px-3 flex justify-between items-center w-full min-h-[48px]">
          <div className="flex items-center gap-2 h-full">
            <Link to="/posts" className="mx-4 text-xl font-bold text-orange-500">ぼざクリ タグ広場</Link>
            <MyLink to="/posts" title="広場" />
            <MyLink to="/deerjikists" title="ニジラー" />
            <MyLink to="/tags" title="タグ" />
            <MyLink to="/wiki/ヘルプ:ホーム" base="/wiki" title="Wiki" />
          </div>
          <div className="ml-auto pr-4">
            <Button onClick={() => setSettingsVisible (true)}>{user?.name || '名もなきニジラー'}</Button>
            <SettingsDialogue visible={settingsVisible}
                              onVisibleChange={setSettingsVisible}
                              user={user}
                              setUser={setUser} />
          </div>
        </nav>
        {(() => {
          const className = 'bg-gray-700 text-white px-3 flex items-center w-full min-h-[40px]'
          const subClass = 'hover:text-orange-500 h-full flex items-center px-3'
          const inputBox = 'flex items-center px-3 mx-2'
          const Separator = () => <span className="flex items-center px-2">|</span>
          switch (selectedMenu)
          {
          case Menu.Post:
            return (
                <div className={className}>
                  <Link to="/posts" className={subClass}>一覧</Link>
                  <Link to="/posts/new" className={subClass}>投稿追加</Link>
                </div>)
          case Menu.Wiki:
            return (
                <div className={className}>
                  <input className={inputBox}
                         placeholder="Wiki 検索" />
                  <Link to="/wiki" className={subClass}>検索</Link>
                  <Link to="/wiki/new" className={subClass}>新規</Link>
                  <Link to="/wiki/changes" className={subClass}>全体履歴</Link>
                  <Link to="/wiki/ヘルプ:Wiki" className={subClass}>ヘルプ</Link>
                  {/^\/wiki\/(?!new|changes)[^\/]+/.test (location.pathname) &&
                      <>
                        <Separator />
                        <Link to={`/posts?tags=${ location.pathname.split ('/')[2] }`} className={subClass}>投稿</Link>
                        <Link to={`/wiki/${ location.pathname.split ('/')[2] }/history`} className={subClass}>履歴</Link>
                        <Link to={`/wiki/${ location.pathname.split ('/')[2] }/edit`} className={subClass}>編輯</Link>
                      </>}
                </div>)
          }
        }) ()}
      </>)
}


export default TopNav
